package com.terraocean.plugin.bridge

import com.terraocean.plugin.Settings
import com.terraocean.plugin.Strings
import com.terraocean.plugin.instance
import hazae41.minecraft.kutils.bukkit.info
import hazae41.minecraft.kutils.bukkit.schedule
import hazae41.minecraft.kutils.bukkit.warning
import okhttp3.*
import org.bukkit.Bukkit
import java.util.concurrent.TimeUnit

val client = OkHttpClient.Builder()
    .readTimeout(0, TimeUnit.MILLISECONDS)
    .build()

private var request = Request.Builder()
    .url(Settings.socketURL)
    .build()
lateinit var socket: WebSocket
var wsFails = 0

fun establishConnection() {
    socket = client.newWebSocket(request, WSListener())
}

fun processMessage(msg: List<String>) {
    when (msg[0].toLowerCase()) {
        "leave" -> {
            Bukkit.getPlayer(msg[1])?.kickPlayer(Strings.kickVoiceChannel)
        }
        "vote" -> {
            Bukkit.getOnlinePlayers().forEach {
                it.sendMessage(Strings.voteNew)
            }
        }
        "votewarn" -> {
            Bukkit.getPlayer(msg[1])?.sendMessage(Strings.voteWarn)
        }
        "votetimeout" -> {
            Bukkit.getPlayer(msg[1])?.kickPlayer(Strings.voteTimeout)
        }
        "kick" -> {
            Bukkit.getPlayer(msg[1])?.kickPlayer(Strings.voteKick)
        }
        "whitelistadd" -> {
            instance.server.run {
                getOfflinePlayer(msg[1]).isWhitelisted = true
            }
        }
        "whitelistremove" -> {
            instance.server.run {
                getPlayer(msg[1])?.kickPlayer(Strings.kickRemovedFromWhitelist)
                getOfflinePlayer(msg[1]).isWhitelisted = false
            }
        }
        else -> instance.warning(
            "Potentially out of date plugin. Unknown message: " + msg.joinToString(" ")
        )
    }
}

class WSListener : WebSocketListener() {
    override fun onMessage(webSocket: WebSocket, text: String) {
        processMessage(text.split(' '))
    }

    override fun onOpen(webSocket: WebSocket, response: Response) {
        instance.info("Socket connection established to $response.")
        wsFails = 0
    }

    override fun onFailure(webSocket: WebSocket, t: Throwable, response: Response?) {
        if (wsFails < 5) {
            wsFails++
            instance.warning("WS Fail: ${t.message}")
            instance.info("Reconnecting to socket...")
        }
        instance.schedule(delay = 5, unit = TimeUnit.SECONDS) {
            establishConnection()
        }
    }
}
